from pathlib import Path

import yaml

import click

from .config import generate_session


class Yaml(dict):
    def __init__(self, data, file_path=None):
        super().__init__(data)
        self.file_path = Path(file_path)
        self.directory = self.file_path.parent


def load_challenge(path):
    try:
        with open(path) as f:
            return Yaml(data=yaml.safe_load(f.read()), file_path=path)
    except FileNotFoundError:
        click.secho(f"No challenge.yml was found in {path}", fg="red")
        return


def load_installed_challenges():
    s = generate_session()
    return s.get("/api/v1/challenges?view=admin", json=True).json()["data"]


def sync_challenge(challenge):
    data = {
        "name": challenge["name"],
        "category": challenge["category"],
        "description": challenge["description"],
        "type": challenge.get("type", "standard"),
        "value": int(challenge["value"]),
    }
    if challenge.get("attempts"):
        data["max_attempts"] = challenge.get("attempts")

    data["state"] = "hidden"

    installed_challenges = load_installed_challenges()
    for c in installed_challenges:
        if c["name"] == challenge["name"]:
            challenge_id = c["id"]
            break
    else:
        return

    s = generate_session()

    original_challenge = s.get(f"/api/v1/challenges/{challenge_id}", json=data).json()[
        "data"
    ]

    r = s.patch(f"/api/v1/challenges/{challenge_id}", json=data)
    r.raise_for_status()

    # Delete existing flags
    current_flags = s.get(f"/api/v1/flags", json=data).json()["data"]
    for flag in current_flags:
        if flag["challenge_id"] == challenge_id:
            flag_id = flag["id"]
            r = s.delete(f"/api/v1/flags/{flag_id}", json=True)
            r.raise_for_status()

    # Create new flags
    if challenge.get("flags"):
        for flag in challenge["flags"]:
            if type(flag) == str:
                data = {"content": flag, "type": "static", "challenge": challenge_id}
                r = s.post(f"/api/v1/flags", json=data)
                r.raise_for_status()
            elif type(flag) == dict:
                flag["challenge"] = challenge_id
                r = s.post(f"/api/v1/flags", json=flag)
                r.raise_for_status()

    # Delete existing tags
    current_tags = s.get(f"/api/v1/tags", json=data).json()["data"]
    for tag in current_tags:
        if tag["challenge_id"] == challenge_id:
            tag_id = tag["id"]
            r = s.delete(f"/api/v1/tags/{tag_id}", json=True)
            r.raise_for_status()

    # Update tags
    if challenge.get("tags"):
        for tag in challenge["tags"]:
            r = s.post(f"/api/v1/tags", json={"challenge": challenge_id, "value": tag})
            r.raise_for_status()

    # Delete existing files
    all_current_files = s.get(f"/api/v1/files?type=challenge", json=data).json()["data"]
    for f in all_current_files:
        for used_file in original_challenge["files"]:
            if f["location"] in used_file:
                file_id = f["id"]
                r = s.delete(f"/api/v1/files/{file_id}", json=True)
                r.raise_for_status()

    # Upload files
    if challenge.get("files"):
        files = []
        for f in challenge["files"]:
            file_path = Path(challenge.directory, f)
            if file_path.exists():
                file_object = ("file", file_path.open(mode="rb"))
                files.append(file_object)
            else:
                click.secho(f"File {file_path} was not found", fg="red")
                raise Exception(f"File {file_path} was not found")

        data = {"challenge": challenge_id, "type": "challenge"}
        # Specifically use data= here instead of json= to send multipart/form-data
        r = s.post(f"/api/v1/files", files=files, data=data)
        r.raise_for_status()

    # Delete existing hints
    current_hints = s.get(f"/api/v1/hints", json=data).json()["data"]
    for hint in current_hints:
        if hint["challenge_id"] == challenge_id:
            hint_id = hint["id"]
            r = s.delete(f"/api/v1/hints/{hint_id}", json=True)
            r.raise_for_status()

    # Create hints
    if challenge.get("hints"):
        for hint in challenge["hints"]:
            if type(hint) == str:
                data = {"content": hint, "cost": 0, "challenge": challenge_id}
            else:
                data = {
                    "content": hint["content"],
                    "cost": hint["cost"],
                    "challenge": challenge_id,
                }

            r = s.post(f"/api/v1/hints", json=data)
            r.raise_for_status()

    # Update requirements
    if challenge.get("requirements"):
        installed_challenges = load_installed_challenges()
        required_challenges = []
        for r in challenge["requirements"]:
            if type(r) == str:
                for c in installed_challenges:
                    if c["name"] == r:
                        required_challenges.append(c["id"])
            elif type(r) == int:
                required_challenges.append(r)

        required_challenges = list(set(required_challenges))
        data = {"requirements": {"prerequisites": required_challenges}}
        r = s.patch(f"/api/v1/challenges/{challenge_id}", json=data)
        r.raise_for_status()

    # Unhide challenge depending upon the value of "state" in spec
    data = {"state": "hidden"}
    if challenge.get("state"):
        if challenge["state"] in ["hidden", "visible"]:
            data["state"] = challenge["state"]

    r = s.patch(f"/api/v1/challenges/{challenge_id}", json=data)
    r.raise_for_status()


def create_challenge(challenge):
    data = {
        "name": challenge["name"],
        "category": challenge["category"],
        "description": challenge["description"],
        "type": challenge.get("type", "standard"),
        "value": int(challenge["value"]),
    }
    if challenge.get("attempts"):
        data["max_attempts"] = challenge.get("attempts")

    s = generate_session()

    r = s.post("/api/v1/challenges", json=data)
    r.raise_for_status()

    challenge_data = r.json()
    challenge_id = challenge_data["data"]["id"]

    # Create flags
    if challenge.get("flags"):
        for flag in challenge["flags"]:
            if type(flag) == str:
                data = {"content": flag, "type": "static", "challenge": challenge_id}
                r = s.post(f"/api/v1/flags", json=data)
                r.raise_for_status()
            elif type(flag) == dict:
                flag["challenge"] = challenge_id
                r = s.post(f"/api/v1/flags", json=flag)
                r.raise_for_status()

    # Create tags
    if challenge.get("tags"):
        for tag in challenge["tags"]:
            r = s.post(f"/api/v1/tags", json={"challenge": challenge_id, "value": tag})
            r.raise_for_status()

    # Upload files
    if challenge.get("files"):
        files = []
        for f in challenge["files"]:
            file_path = Path(challenge.directory, f)
            if file_path.exists():
                file_object = ("file", file_path.open(mode="rb"))
                files.append(file_object)
            else:
                click.secho(f"File {file_path} was not found", fg="red")
                raise Exception(f"File {file_path} was not found")

        data = {"challenge": challenge_id, "type": "challenge"}
        # Specifically use data= here instead of json= to send multipart/form-data
        r = s.post(f"/api/v1/files", files=files, data=data)
        r.raise_for_status()

    # Add hints
    if challenge.get("hints"):
        for hint in challenge["hints"]:
            if type(hint) == str:
                data = {"content": hint, "cost": 0, "challenge": challenge_id}
            else:
                data = {
                    "content": hint["content"],
                    "cost": hint["cost"],
                    "challenge": challenge_id,
                }

            r = s.post(f"/api/v1/hints", json=data)
            r.raise_for_status()

    # Add requirements
    if challenge.get("requirements"):
        installed_challenges = load_installed_challenges()
        required_challenges = []
        for r in challenge["requirements"]:
            if type(r) == str:
                for c in installed_challenges:
                    if c["name"] == r:
                        required_challenges.append(c["id"])
            elif type(r) == int:
                required_challenges.append(r)

        required_challenges = list(set(required_challenges))
        data = {"requirements": {"prerequisites": required_challenges}}
        r = s.patch(f"/api/v1/challenges/{challenge_id}", json=data)
        r.raise_for_status()

    # Set challenge state
    if challenge.get("state"):
        data = {"state": "hidden"}
        if challenge["state"] in ["hidden", "visible"]:
            data["state"] = challenge["state"]

        r = s.patch(f"/api/v1/challenges/{challenge_id}", json=data)
        r.raise_for_status()


def lint_challenge(path):
    try:
        challenge = load_challenge(path)
    except yaml.YAMLError as e:
        click.secho(f"Error parsing challenge.yml: {e}", fg="red")
        exit(1)

    required_fields = ["name", "author", "category", "description", "value"]
    errors = []
    for field in required_fields:
        if challenge.get(field) is None:
            errors.append(field)

    if len(errors) > 0:
        print("Missing fields: ", ", ".join(errors))
        exit(1)

    exit(0)



#!/usr/bin/env python
import random

from six.moves import input


def server():
    print("Hello World")
    secret = random.randint(1, 100)
    while True:
        guess = input("What's the random number?")
        if int(guess) == secret:
            print("You got it!")
            exit()
        else:
            print("No.")


if __name__ == "__main__":
    server()



import os
import subprocess

import click

from pygments import highlight
from pygments.formatters import TerminalFormatter
from pygments.lexers import IniLexer, JsonLexer
from ctfcli.utils.config import get_config_path, preview_config


class Config(object):
    def edit(self):
        editor = os.getenv("EDITOR", "vi")
        command = editor, get_config_path()
        subprocess.call(command)

    def path(self):
        click.echo(get_config_path())

    def view(self, color=True, json=False):
        config = get_config_path()
        with open(config) as f:
            if json is True:
                config = preview_config(as_string=True)
                if color:
                    config = highlight(config, JsonLexer(), TerminalFormatter())
            else:
                config = f.read()
                if color:
                    config = highlight(config, IniLexer(), TerminalFormatter())

            print(config)



import os
import shutil
import subprocess
from cookiecutter.main import cookiecutter
from pathlib import Path

import click
import yaml

from ctfcli.utils.challenge import (
    create_challenge,
    lint_challenge,
    load_challenge,
    load_installed_challenges,
    sync_challenge,
)
from ctfcli.utils.config import (
    get_base_path,
    get_config_path,
    get_project_path,
    load_config,
)
from ctfcli.utils.spec import (
    CHALLENGE_SPEC_DOCS,
    blank_challenge_spec,
)


class Challenge(object):
    def new(self, type):
        path = Path(get_base_path())
        if os.sep not in type:
            type += os.sep + "default"
        path = path / "templates" / type
        cookiecutter(str(path))

    def add(self, repo):
        config = load_config()

        if repo.endswith(".git"):
            # Get relative path from project root to current directory
            challenge_path = Path(os.path.relpath(os.getcwd(), get_project_path()))

            # Get new directory that will exist after clone
            base_repo_path = Path(os.path.basename(repo).rsplit(".", maxsplit=1)[0])

            # Join targets
            challenge_path = challenge_path / base_repo_path
            print(challenge_path)

            config["challenges"][str(challenge_path)] = repo

            with open(get_config_path(), "w+") as f:
                config.write(f)

            subprocess.call(["git", "clone", "--depth", "1", repo])
            shutil.rmtree(str(base_repo_path / ".git"))
        elif Path(repo).exists():
            config["challenges"][repo] = repo
            with open(get_config_path(), "w+") as f:
                config.write(f)
        else:
            click.secho(
                "Couldn't process that challenge path. Please check it for errors.",
                fg="red",
            )

    def restore(self, challenge=None):
        config = load_config()
        challenges = dict(config["challenges"])
        for folder, url in challenges.items():
            if url.endswith(".git"):
                if challenge is not None and folder != challenge:
                    continue
                click.echo(f"Cloning {url} to {folder}")
                subprocess.call(["git", "clone", "--depth", "1", url, folder])
                shutil.rmtree(str(Path(folder) / ".git"))
            else:
                click.echo(f"Skipping {url} - {folder}")

    def install(self, challenge=None, force=False):
        if challenge is None:
            challenge = os.getcwd()

        path = Path(challenge)

        if path.name.endswith(".yml") is False:
            path = path / "challenge.yml"

        click.secho(f"Found {path}")
        challenge = load_challenge(path)
        click.secho(f'Loaded {challenge["name"]}', fg="yellow")

        installed_challenges = load_installed_challenges()
        for c in installed_challenges:
            if c["name"] == challenge["name"]:
                click.secho(
                    "Already found existing challenge with same name. Perhaps you meant sync instead of install?",
                    fg="red",
                )
                if force is True:
                    click.secho(
                        "Ignoring existing challenge because of --force", fg="yellow"
                    )
                else:
                    return

        click.secho(f'Installing {challenge["name"]}', fg="yellow")
        create_challenge(challenge=challenge)
        click.secho(f"Success!", fg="green")

    def sync(self, challenge=None):
        if challenge is None:
            challenge = os.getcwd()

        path = Path(challenge)

        if path.name.endswith(".yml") is False:
            path = path / "challenge.yml"

        click.secho(f"Found {path}")
        challenge = load_challenge(path)
        click.secho(f'Loaded {challenge["name"]}', fg="yellow")

        installed_challenges = load_installed_challenges()
        for c in installed_challenges:
            if c["name"] == challenge["name"]:
                break
        else:
            click.secho(
                "Couldn't find existing challenge. Perhaps you meant install instead of sync?",
                fg="red",
            )

        click.secho(f'Syncing {challenge["name"]}', fg="yellow")
        sync_challenge(challenge=challenge)
        click.secho(f"Success!", fg="green")

    def update(self):
        config = load_config()
        challenges = dict(config["challenges"])
        for folder, url in challenges.items():
            if url.endswith(".git"):
                click.echo(f"Cloning {url} to {folder}")
                subprocess.call(["git", "init"], cwd=folder)
                subprocess.call(["git", "remote", "add", "origin", url], cwd=folder)
                subprocess.call(["git", "pull"], cwd=folder)
                shutil.rmtree(str(Path(folder) / ".git"))
            else:
                click.echo(f"Skipping {url} - {folder}")

    def finalize(self, challenge=None):
        if challenge is None:
            challenge = os.getcwd()

        path = Path(challenge)
        spec = blank_challenge_spec()
        for k in spec:
            q = CHALLENGE_SPEC_DOCS.get(k)
            fields = q._asdict()

            ask = False
            required = fields.pop("required", False)
            if required is False:
                try:
                    ask = click.confirm(f"Would you like to add the {k} field?")
                    if ask is False:
                        continue
                except click.Abort:
                    click.echo("\n")
                    continue

            if ask is True:
                fields["text"] = "\t" + fields["text"]

            multiple = fields.pop("multiple", False)
            if multiple:
                fields["text"] += " (Ctrl-C to continue)"
                spec[k] = []
                try:
                    while True:
                        r = click.prompt(**fields)
                        spec[k].append(r)
                except click.Abort:
                    click.echo("\n")
            else:
                try:
                    r = click.prompt(**fields)
                    spec[k] = r
                except click.Abort:
                    click.echo("\n")

        with open(path / "challenge.yml", "w+") as f:
            yaml.dump(spec, stream=f, default_flow_style=False, sort_keys=False)

        print("challenge.yml written to", path / "challenge.yml")

    def lint(self, challenge=None):
        if challenge is None:
            challenge = os.getcwd()

        path = Path(challenge)

        if path.name.endswith(".yml") is False:
            path = path / "challenge.yml"

        lint_challenge(path)



import os
import shutil
import subprocess

from ctfcli.utils.plugins import get_plugin_dir


class Plugins(object):
    def install(self, url):
        local_dir = os.path.join(
            get_plugin_dir(), os.path.basename(url).rsplit(".", maxsplit=1)[0]
        )
        subprocess.call(["git", "clone", url, local_dir])
        subprocess.call(
            ["pip", "install", "-r", os.path.join(local_dir, "requirements.txt")]
        )

    def uninstall(self, plugin_name):
        plugin_dir = os.path.join(get_plugin_dir(), plugin_name)
        shutil.rmtree(plugin_dir)

    def list(self):
        for plugin in sorted(os.listdir(get_plugin_dir())):
            print(plugin)

    def dir(self):
        print(get_plugin_dir())



import appdirs
import os

from ctfcli import __name__ as pkg_name


def get_plugin_dir():
    plugins_path = os.path.join(get_data_dir(), "plugins")
    if not os.path.exists(plugins_path):
        os.makedirs(plugins_path)
    return os.path.join(plugins_path)


def get_data_dir():
    return appdirs.user_data_dir(appname=pkg_name)



#!/usr/bin/env python
import codecs


def encrypt(string):
    enc = codecs.getencoder("rot-13")
    return enc(string)[0]


def main():
    with open("src/flag.txt") as f:
        print(encrypt(f.read()))


if __name__ == "__main__":
    main()



import configparser
import importlib
import os
import subprocess
import sys
from pathlib import Path

import click

from ctfcli.cli.challenges import Challenge
from ctfcli.cli.config import Config
from ctfcli.cli.plugins import Plugins
from ctfcli.utils.plugins import get_plugin_dir

import fire


class CTFCLI(object):
    def init(self):
        ctf_url = click.prompt("Please enter CTFd instance URL")
        ctf_token = click.prompt("Please enter CTFd Admin Access Token")
        if (
            click.confirm(f"Do you want to continue with {ctf_url} and {ctf_token}")
            is False
        ):
            click.echo("Aborted!")
            return

        if Path(".ctf").exists():
            click.secho(".ctf/ folder already exists. Aborting!", fg="red")
            return

        os.mkdir(".ctf")

        config = configparser.ConfigParser()
        config["config"] = {"url": ctf_url, "access_token": ctf_token}
        config["challenges"] = {}

        with open(".ctf/config", "a+") as f:
            config.write(f)

        subprocess.call(["git", "init"])

    def config(self):
        return COMMANDS.get("config")

    def challenge(self):
        return COMMANDS.get("challenge")

    def plugins(self):
        return COMMANDS.get("plugins")


COMMANDS = {
    "challenge": Challenge(),
    "config": Config(),
    "plugins": Plugins(),
    "cli": CTFCLI(),
}


def main():
    # Load plugins
    plugin_dir = get_plugin_dir()
    sys.path.insert(0, plugin_dir)
    for plugin in sorted(os.listdir(plugin_dir)):
        plugin_path = os.path.join(plugin_dir, plugin, "__init__.py")
        print("Loading", plugin_path, "as", plugin)
        loaded = importlib.import_module(plugin)
        loaded.load(COMMANDS)
    sys.path.remove(plugin_dir)

    # Load CLI
    fire.Fire(CTFCLI)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
from flask import Flask, render_template
from flask_bootstrap import Bootstrap

app = Flask(__name__)
app.config["BOOTSTRAP_SERVE_LOCAL"] = True
bootstrap = Bootstrap(app)


@app.route("/")
def index():
    return render_template("index.html")


if __name__ == "__main__":
    app.run(debug=True, threaded=True)



from urllib.parse import urljoin

from requests import Session


class APISession(Session):
    def __init__(self, prefix_url=None, *args, **kwargs):
        super(APISession, self).__init__(*args, **kwargs)
        self.prefix_url = prefix_url

    def request(self, method, url, *args, **kwargs):
        url = urljoin(self.prefix_url, url)
        return super(APISession, self).request(method, url, *args, **kwargs)



import yaml

from collections import namedtuple
from pathlib import Path
import getpass


Prompt = namedtuple("Prompt", ["text", "type", "default", "required", "multiple"])


CHALLENGE_SPEC_DOCS = {
    "name": Prompt(
        text="Challenge name or identifier",
        type=None,
        default=None,
        required=True,
        multiple=False,
    ),
    "author": Prompt(
        text="Your name or handle",
        type=None,
        default=getpass.getuser(),
        required=True,
        multiple=False,
    ),
    "category": Prompt(
        text="Challenge category",
        type=None,
        default=None,
        required=True,
        multiple=False,
    ),
    "description": Prompt(
        text="Challenge description shown to the user",
        type=None,
        default=None,
        required=True,
        multiple=False,
    ),
    "value": Prompt(
        text="How many points your challenge should be worth",
        type=int,
        default=None,
        required=True,
        multiple=False,
    ),
    "version": Prompt(
        text="What version of the challenge specification was used",
        type=None,
        default="0.1",
        required=False,
        multiple=False,
    ),
    "image": Prompt(
        text="Docker image used to deploy challenge",
        type=None,
        default=None,
        required=False,
        multiple=False,
    ),
    "type": Prompt(
        text="Type of challenge",
        type=None,
        default="standard",
        required=True,
        multiple=False,
    ),
    "attempts": Prompt(
        text="How many attempts should the player have",
        type=int,
        default=None,
        required=False,
        multiple=False,
    ),
    "flags": Prompt(
        text="Flags that mark the challenge as solved",
        type=None,
        default=None,
        required=False,
        multiple=True,
    ),
    "tags": Prompt(
        text="Tag that denotes a challenge topic",
        type=None,
        default=None,
        required=False,
        multiple=True,
    ),
    "files": Prompt(
        text="Files to be shared with the user",
        type=None,
        default=None,
        required=False,
        multiple=True,
    ),
    "hints": Prompt(
        text="Hints to be shared with the user",
        type=None,
        default=None,
        required=False,
        multiple=True,
    ),
    "requirements": Prompt(
        text="Challenge dependencies that must be solved before this one can be attempted",
        type=None,
        default=None,
        required=False,
        multiple=True,
    ),
}


def blank_challenge_spec():
    pwd = Path(__file__)
    spec = pwd.parent.parent / "spec" / "challenge-example.yml"
    with open(spec) as f:
        blank = yaml.safe_load(f)

    for k in blank:
        if k != "version":
            blank[k] = None

    return blank



import configparser
import json
import os

from .api import APISession

from ctfcli import __file__ as base_path


def get_base_path():
    return os.path.dirname(base_path)


def get_config_path():
    pwd = os.getcwd()
    while pwd:
        config = os.path.join(pwd, ".ctf/config")
        if os.path.isfile(config):
            return config
        new_pwd = os.path.dirname(pwd)
        pwd = None if new_pwd == pwd else new_pwd
    return None


def get_project_path():
    pwd = os.getcwd()
    while pwd:
        config = os.path.join(pwd, ".ctf/config")
        if os.path.isfile(config):
            return pwd
        new_pwd = os.path.dirname(pwd)
        pwd = None if new_pwd == pwd else new_pwd
    return None


def load_config():
    path = get_config_path()
    parser = configparser.ConfigParser()

    # Preserve case in configparser
    parser.optionxform = str

    parser.read(path)
    return parser


def preview_config(as_string=False):
    config = load_config()

    d = {}
    for section in config.sections():
        d[section] = {}
        for k, v in config.items(section):
            d[section][k] = v

    preview = json.dumps(d, sort_keys=True, indent=4)

    if as_string is True:
        return preview
    else:
        print(preview)


def generate_session():
    config = load_config()
    url = config["config"]["url"]
    url = url.strip("/")
    access_token = config["config"]["access_token"]
    s = APISession(url)
    s.headers.update({"Authorization": f"Token {access_token}"})
    return s



# -*- coding: utf-8 -*-
import re

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

import os

with open("ctfcli/__init__.py") as f:
    version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)


def read(fname):
    try:
        with open(os.path.join(os.path.dirname(__file__), fname), "r") as fp:
            return fp.read().strip()
    except IOError:
        return ""


setup(
    name="ctfcli",
    version=version,
    author="Kevin Chung",
    author_email="kchung@ctfd.io",
    license="Apache 2.0",
    description="Tool for creating and running Capture The Flag competitions",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    keywords=["ctf"],
    classifiers=[],
    install_requires=[
        "cookiecutter==1.6.0",
        "click==7.0",
        "fire==0.2.1",
        "pyyaml==5.2",
        "Pygments==2.5.2",
        "requests==2.22.0",
        "colorama==0.4.3",
        "appdirs==1.4.3",
    ],
    packages=find_packages(),
    include_package_data=True,
    entry_points={"console_scripts": ["ctf = ctfcli.__main__:main"]},
)
